@@ -4,7 +4,13 @@
 const char* ffDetectMemory(FFMemoryResult* ram)
 {
     size_t length = sizeof(ram->bytesTotal);
-    if (sysctl((int[]){ CTL_HW, HW_PHYSMEM }, 2, &ram->bytesTotal, &length, NULL, 0))
+    if (sysctl((int[]){ CTL_HW,
+#if __NetBSD__
+        HW_PHYSMEM64
+#else
+        HW_PHYSMEM
+#endif
+    }, 2, &ram->bytesTotal, &length, NULL, 0))
         return "Failed to read hw.physmem";
 
     // vm.stats.vm.* are int values
@@ -12,7 +18,7 @@ const char* ffDetectMemory(FFMemoryResult* ram)
         + ffSysctlGetInt("vm.stats.vm.v_inactive_count", 0)
         + ffSysctlGetInt("vm.stats.vm.v_cache_count", 0);
 
-    ram->bytesUsed = ram->bytesTotal - (uint64_t) pagesFree * instance.state.platform.pageSize;
+    ram->bytesUsed = ram->bytesTotal - (uint64_t) pagesFree * instance.state.platform.sysinfo.pageSize;
 
     return NULL;
 }
