@@ -6,8 +6,6 @@
 #include "modules/cpu/cpu.h"
 #include "util/stringUtils.h"
 
-#define FF_CPU_NUM_FORMAT_ARGS 10
-
 static int sortCores(const FFCPUCore* a, const FFCPUCore* b)
 {
     return (int)b->freq - (int)a->freq;
@@ -17,10 +15,8 @@ void ffPrintCPU(FFCPUOptions* options)
 {
     FFCPUResult cpu = {
         .temperature = FF_CPU_TEMP_UNSET,
-        .frequencyMin = 0.0/0.0,
-        .frequencyMax = 0.0/0.0,
-        .frequencyBase = 0.0/0.0,
-        .frequencyBiosLimit = 0.0/0.0,
+        .frequencyMax = 0,
+        .frequencyBase = 0,
         .name = ffStrbufCreate(),
         .vendor = ffStrbufCreate(),
     };
@@ -41,7 +37,7 @@ void ffPrintCPU(FFCPUOptions* options)
         if (options->showPeCoreCount)
         {
             uint32_t typeCount = 0;
-            while (cpu.coreTypes[typeCount].count != 0 && typeCount < sizeof(cpu.coreTypes) / sizeof(cpu.coreTypes[0])) typeCount++;
+            while (cpu.coreTypes[typeCount].count != 0 && typeCount < ARRAY_SIZE(cpu.coreTypes)) typeCount++;
             if (typeCount > 0)
             {
                 qsort(cpu.coreTypes, typeCount, sizeof(cpu.coreTypes[0]), (void*) sortCores);
@@ -57,6 +53,9 @@ void ffPrintCPU(FFCPUOptions* options)
 
             FF_STRBUF_AUTO_DESTROY str = ffStrbufCreate();
 
+            if(cpu.packages > 1)
+                ffStrbufAppendF(&str, "%u x ", cpu.packages);
+
             if(cpu.name.length > 0)
                 ffStrbufAppend(&str, &cpu.name);
             else if(cpu.vendor.length > 0)
@@ -70,15 +69,21 @@ void ffPrintCPU(FFCPUOptions* options)
             if(coreTypes.length > 0)
                 ffStrbufAppendF(&str, " (%s)", coreTypes.chars);
             else if(cpu.coresOnline > 1)
-                ffStrbufAppendF(&str, " (%u)", cpu.coresOnline);
+            {
+                if(cpu.packages > 1)
+                    ffStrbufAppendF(&str, " (%u)", cpu.coresOnline / 2);
+                else
+                    ffStrbufAppendF(&str, " (%u)", cpu.coresOnline);
+            }
 
-            double freq = cpu.frequencyBiosLimit;
-            if(!(freq > 0.0000001))
-                freq = cpu.frequencyMax;
-            if(!(freq > 0.0000001))
+            uint32_t freq = cpu.frequencyMax;
+            if(freq == 0)
                 freq = cpu.frequencyBase;
-            if(freq > 0.0000001)
-                ffStrbufAppendF(&str, " @ %.*f GHz", options->freqNdigits, freq);
+            if(freq > 0)
+            {
+                ffStrbufAppendS(&str, " @ ");
+                ffParseFrequency(freq, &str);
+            }
 
             if(cpu.temperature == cpu.temperature) //FF_CPU_TEMP_UNSET
             {
@@ -90,33 +95,24 @@ void ffPrintCPU(FFCPUOptions* options)
         }
         else
         {
-            char freqBase[32], freqMax[32], freqBioslimit[32];
-            if (cpu.frequencyBase > 0)
-                snprintf(freqBase, sizeof(freqBase), "%.*f", options->freqNdigits, cpu.frequencyBase);
-            else
-                freqBase[0] = 0;
-            if (cpu.frequencyMax > 0)
-                snprintf(freqMax, sizeof(freqMax), "%.*f", options->freqNdigits, cpu.frequencyMax);
-            else
-                freqMax[0] = 0;
-            if (cpu.frequencyBiosLimit > 0)
-                snprintf(freqBioslimit, sizeof(freqBioslimit), "%.*f", options->freqNdigits, cpu.frequencyBiosLimit);
-            else
-                freqBioslimit[0] = 0;
+            FF_STRBUF_AUTO_DESTROY freqBase = ffStrbufCreate();
+            ffParseFrequency(cpu.frequencyBase, &freqBase);
+            FF_STRBUF_AUTO_DESTROY freqMax = ffStrbufCreate();
+            ffParseFrequency(cpu.frequencyMax, &freqMax);
 
             FF_STRBUF_AUTO_DESTROY tempStr = ffStrbufCreate();
             ffTempsAppendNum(cpu.temperature, &tempStr, options->tempConfig, &options->moduleArgs);
-            FF_PRINT_FORMAT_CHECKED(FF_CPU_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, FF_CPU_NUM_FORMAT_ARGS, ((FFformatarg[]){
-                {FF_FORMAT_ARG_TYPE_STRBUF, &cpu.name, "name"},
-                {FF_FORMAT_ARG_TYPE_STRBUF, &cpu.vendor, "vendor"},
-                {FF_FORMAT_ARG_TYPE_UINT16, &cpu.coresPhysical, "cores-physical"},
-                {FF_FORMAT_ARG_TYPE_UINT16, &cpu.coresLogical, "cores-logical"},
-                {FF_FORMAT_ARG_TYPE_UINT16, &cpu.coresOnline, "cores-online"},
-                {FF_FORMAT_ARG_TYPE_STRING, freqBase, "freq-base"},
-                {FF_FORMAT_ARG_TYPE_STRING, freqMax, "freq-max"},
-                {FF_FORMAT_ARG_TYPE_STRBUF, &tempStr, "temperature"},
-                {FF_FORMAT_ARG_TYPE_STRBUF, &coreTypes, "core-types"},
-                {FF_FORMAT_ARG_TYPE_STRING, freqBioslimit, "freq-bios-limit"},
+            FF_PRINT_FORMAT_CHECKED(FF_CPU_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, ((FFformatarg[]){
+                FF_FORMAT_ARG(cpu.name, "name"),
+                FF_FORMAT_ARG(cpu.vendor, "vendor"),
+                FF_FORMAT_ARG(cpu.coresPhysical, "cores-physical"),
+                FF_FORMAT_ARG(cpu.coresLogical, "cores-logical"),
+                FF_FORMAT_ARG(cpu.coresOnline, "cores-online"),
+                FF_FORMAT_ARG(freqBase, "freq-base"),
+                FF_FORMAT_ARG(freqMax, "freq-max"),
+                FF_FORMAT_ARG(tempStr, "temperature"),
+                FF_FORMAT_ARG(coreTypes, "core-types"),
+                FF_FORMAT_ARG(cpu.packages, "packages"),
             }));
         }
     }
@@ -135,12 +131,6 @@ bool ffParseCPUCommandOptions(FFCPUOptions* options, const char* key, const char
     if (ffTempsParseCommandOptions(key, subKey, value, &options->temp, &options->tempConfig))
         return true;
 
-    if (ffStrEqualsIgnCase(subKey, "freq-ndigits"))
-    {
-        options->freqNdigits = (uint8_t) ffOptionParseUInt32(key, value);
-        return true;
-    }
-
     if (ffStrEqualsIgnCase(subKey, "show-pe-core-count"))
     {
         options->showPeCoreCount = ffOptionParseBoolean(value);
@@ -168,7 +158,7 @@ void ffParseCPUJsonObject(FFCPUOptions* options, yyjson_val* module)
 
         if (ffStrEqualsIgnCase(key, "freqNdigits"))
         {
-            options->freqNdigits = (uint8_t) yyjson_get_uint(val);
+            ffPrintError(FF_CPU_MODULE_NAME, 0, &options->moduleArgs, FF_PRINT_TYPE_DEFAULT, "modules.CPU.freqNdigits has been moved to display.freq.ndigits");
             continue;
         }
 
@@ -191,9 +181,6 @@ void ffGenerateCPUJsonConfig(FFCPUOptions* options, yyjson_mut_doc* doc, yyjson_
 
     ffTempsGenerateJsonConfig(doc, module, defaultOptions.temp, defaultOptions.tempConfig, options->temp, options->tempConfig);
 
-    if (defaultOptions.freqNdigits != options->freqNdigits)
-        yyjson_mut_obj_add_uint(doc, module, "freqNdigits", options->freqNdigits);
-
     if (defaultOptions.showPeCoreCount != options->showPeCoreCount)
         yyjson_mut_obj_add_bool(doc, module, "showPeCoreCount", options->showPeCoreCount);
 }
@@ -202,10 +189,8 @@ void ffGenerateCPUJsonResult(FFCPUOptions* options, yyjson_mut_doc* doc, yyjson_
 {
     FFCPUResult cpu = {
         .temperature = FF_CPU_TEMP_UNSET,
-        .frequencyMin = 0.0/0.0,
-        .frequencyMax = 0.0/0.0,
-        .frequencyBase = 0.0/0.0,
-        .frequencyBiosLimit = 0.0/0.0,
+        .frequencyMax = 0,
+        .frequencyBase = 0,
         .name = ffStrbufCreate(),
         .vendor = ffStrbufCreate(),
     };
@@ -225,6 +210,10 @@ void ffGenerateCPUJsonResult(FFCPUOptions* options, yyjson_mut_doc* doc, yyjson_
         yyjson_mut_val* obj = yyjson_mut_obj_add_obj(doc, module, "result");
         yyjson_mut_obj_add_strbuf(doc, obj, "cpu", &cpu.name);
         yyjson_mut_obj_add_strbuf(doc, obj, "vendor", &cpu.vendor);
+        if (cpu.packages == 0)
+            yyjson_mut_obj_add_null(doc, obj, "packages");
+        else
+            yyjson_mut_obj_add_uint(doc, obj, "packages", cpu.packages);
 
         yyjson_mut_val* cores = yyjson_mut_obj_add_obj(doc, obj, "cores");
         yyjson_mut_obj_add_uint(doc, cores, "physical", cpu.coresPhysical);
@@ -232,10 +221,8 @@ void ffGenerateCPUJsonResult(FFCPUOptions* options, yyjson_mut_doc* doc, yyjson_
         yyjson_mut_obj_add_uint(doc, cores, "online", cpu.coresOnline);
 
         yyjson_mut_val* frequency = yyjson_mut_obj_add_obj(doc, obj, "frequency");
-        yyjson_mut_obj_add_real(doc, frequency, "base", cpu.frequencyBase);
-        yyjson_mut_obj_add_real(doc, frequency, "max", cpu.frequencyMax);
-        yyjson_mut_obj_add_real(doc, frequency, "min", cpu.frequencyMin);
-        yyjson_mut_obj_add_real(doc, frequency, "biosLimit", cpu.frequencyBiosLimit);
+        yyjson_mut_obj_add_uint(doc, frequency, "base", cpu.frequencyBase);
+        yyjson_mut_obj_add_uint(doc, frequency, "max", cpu.frequencyMax);
 
         yyjson_mut_val* coreTypes = yyjson_mut_obj_add_arr(doc, obj, "coreTypes");
         for (uint32_t i = 0; i < sizeof (cpu.coreTypes) / sizeof (cpu.coreTypes[0]) && cpu.coreTypes[i].count > 0; i++)
@@ -252,39 +239,34 @@ void ffGenerateCPUJsonResult(FFCPUOptions* options, yyjson_mut_doc* doc, yyjson_
     ffStrbufDestroy(&cpu.vendor);
 }
 
-void ffPrintCPUHelpFormat(void)
-{
-    FF_PRINT_MODULE_FORMAT_HELP_CHECKED(FF_CPU_MODULE_NAME, "{1} ({5}) @ {7} GHz", FF_CPU_NUM_FORMAT_ARGS, ((const char* []) {
-        "Name - name",
-        "Vendor - vendor",
-        "Physical core count - cores-physical",
-        "Logical core count - cores-logical",
-        "Online core count - cores-online",
-        "Base frequency - freq-base",
-        "Max frequency - freq-max",
-        "Temperature (formatted) - temperature",
-        "Logical core count grouped by frequency - core-types",
-        "Bios limited frequency - freq-bios-limit",
-    }));
-}
+static FFModuleBaseInfo ffModuleInfo = {
+    .name = FF_CPU_MODULE_NAME,
+    .description = "Print CPU name, frequency, etc",
+    .parseCommandOptions = (void*) ffParseCPUCommandOptions,
+    .parseJsonObject = (void*) ffParseCPUJsonObject,
+    .printModule = (void*) ffPrintCPU,
+    .generateJsonResult = (void*) ffGenerateCPUJsonResult,
+    .generateJsonConfig = (void*) ffGenerateCPUJsonConfig,
+    .formatArgs = FF_FORMAT_ARG_LIST(((FFModuleFormatArg[]) {
+        {"Name", "name"},
+        {"Vendor", "vendor"},
+        {"Physical core count", "cores-physical"},
+        {"Logical core count", "cores-logical"},
+        {"Online core count", "cores-online"},
+        {"Base frequency (formatted)", "freq-base"},
+        {"Max frequency (formatted)", "freq-max"},
+        {"Temperature (formatted)", "temperature"},
+        {"Logical core count grouped by frequency", "core-types"},
+        {"Processor package count", "packages"},
+    }))
+};
 
 void ffInitCPUOptions(FFCPUOptions* options)
 {
-    ffOptionInitModuleBaseInfo(
-        &options->moduleInfo,
-        FF_CPU_MODULE_NAME,
-        "Print CPU name, frequency, etc",
-        ffParseCPUCommandOptions,
-        ffParseCPUJsonObject,
-        ffPrintCPU,
-        ffGenerateCPUJsonResult,
-        ffPrintCPUHelpFormat,
-        ffGenerateCPUJsonConfig
-    );
-    ffOptionInitModuleArg(&options->moduleArgs);
+    options->moduleInfo = ffModuleInfo;
+    ffOptionInitModuleArg(&options->moduleArgs, "");
     options->temp = false;
     options->tempConfig = (FFColorRangeConfig) { 60, 80 };
-    options->freqNdigits = 2;
     options->showPeCoreCount = false;
 }
 
